/******************************************************************************
* NORTi Network TvCvO                                      *
*  (MiNET-7042, MS7615SE, MS7709SE, MS7709ASE, MS7727SE, MS7750SEp)        *
*  (SH7615 EVA BOD+)                                                          *
*                                                                             *
*  Copyright (c) 1998-2003, MiSPO Co., Ltd.                                   *
*  All rights reserved.                                                       *
*                                                                             *
* 23/Apr/2000  Created                                                        *
* 06/Apr/2000  Supported MS7615SE01 board                                     *
* 20/Jun/2001  Supported MS7727SE01 board                                     *
* 08/Feb/2001  Supported SH7615 EVA BOD+                                      *
* 16/Feb/2003  Supported NORTi Simulator                                   SK *
******************************************************************************/

#include <stdio.h>
#include <string.h>
#include "kernel.h"
#include "nosio.h"

#if !defined(NORTiSIM)

#if defined(MS7709A)
#include "nos7709a.h"
#include "sh7709a.h"
#elif defined(MS7709)
#include "nos7709.h"
#include "sh7709.h"
#elif defined(MiNET7042)
#include "net7042.h"
#elif defined(MS7750)
#include "nos7750.h"
#include "sh7750.h"
#elif defined(MS7615) || defined(SH7615EV)
#include "nos7615.h"
#include "sh7615.h"
#endif

#endif

#include "nonet.h"
#include "nonecho.h"
#include "nontftp.h"
#include "nonteln.h"
#include "noncons.h"
#include "nonfile.h"
#include "nonftp.h"
#include "nonitod.h"
#include "nondhcp.h"

#ifdef MiNET7042
const char title[] = "\r\n*** MiNET-7042 Test Program Ver 2.0 ***";
#endif

/* ubN */

T_TELNETD telnetd;              /* TELNETD ubN */
T_CONSOLE console;              /* Console ubN */
T_FTP ftp;                      /* FTP Server ubN */
T_FTP ftpc;                     /* FTP Client ubN */
#define NFILE           8       /* I[v\ȃt@C */
T_FILE file[NFILE];             /* t@CubN */
T_DISK disk[1];                 /* fBXNhCuubN */

/* IuWFNg */

TASK MainTask(void);
const T_CTSK cMainTask = { TA_HLNG, NULL, MainTask, 7, 1024, NULL, (B *)"MainTask" };

/*****************************************************************************
* TELNET ̃pX[h`FbN
*
******************************************************************************/

B *telnet_passwd_check(T_TELNETD *t, const B *name, const B *passwd)
{
  #if 0
    if (strcmp(name, "mispo") == 0
     && strcmp(passwd, "12345") == 0)
        return (B *)"mispo>";
    else
        return NULL;
  #else
    return (B *)">";
  #endif
}

/*****************************************************************************
* TELNET ̃R}h
*
******************************************************************************/

int ping_command(int argc, char *argv[]);
int ftp_command(int argc, char *argv[]);
int dial_command(int argc, char *argv[]);
int telnet_command(int argc, char *argv[]);
int dns_command(int argc, char *argv[]);

BOOL telnetd_callback(T_TERMINAL *t, B *s)
{
    char *argv[16];
    int argc, r;

    for (argc = 0; argc < sizeof argv / sizeof (B *); argc++)
    {   while (*s == ' ')
            s++;
        if (*s == '\0')
            break;
        argv[argc] = (char *)s;
        s = (B *)strchr((char *)s, ' ');
        if (s == NULL)
            break;
        *s++ = '\0';
    }
    argc++;

    if (stricmp(argv[0], (char *)"?") == 0 || stricmp(argv[0], (char *)"help") == 0)
#ifdef MiNET7042
        print((B *)"test  ping  ftp  telnet  dns  dhcp");
#else
        print((B *)"ip  ping  ftp  telnet  techo  uecho  dns  dhcp");
#endif
    else if (stricmp(argv[0], (char *)"ping") == 0)
        r = ping_command(argc, argv);
    else if (stricmp(argv[0], (char *)"ftp") == 0)
        r = ftp_command(argc, argv);
    else if (stricmp(argv[0], (char *)"telnet") == 0)
        r = telnet_command(argc, argv);
    else if (stricmp(argv[0], (char *)"dns") == 0)
        dns_command(argc, argv);
    else if (stricmp(argv[0], (char *)"dhcp") == 0)
        dhcp_command();
#ifndef MiNET7042
    else if (stricmp(argv[0], (char *)"techo") == 0)
        tcp_echo_client(0);
    else if (stricmp(argv[0], (char *)"uecho") == 0)
        udp_echo_client(0);
    else if (stricmp(argv[0], (char *)"ip") == 0)
        disp_param();
#else
    else if (stricmp(argv[0], (char *)"test") == 0)
        r = test_command(argc, argv);
#endif

    else
        ; 

    return TRUE;
}

/*****************************************************************************
* \⏕֐
*
******************************************************************************/

static void print_ip(UB *ipaddr)
{
    B  buf[10];
    INT i;

    for (i = 0 ; ; i++) {
        itou((char *)buf, (UH)ipaddr[i], itoulen(ipaddr[i]));
        print(buf);
        if (i >= 3)
            break;
        print((B *)".");
    }
}

static void print_mac(UB *macaddr)
{
    B  buf[10];
    INT i;

    for (i = 0 ; ; i++) {
        ito0X((char *)buf, (UH)macaddr[i], 2);
        print(buf);
        if (i >= 5)
            break;
        print((B *)"-");
    }
}

void error(char *err_str, ER ercd)
{
    char str[10];

    ito_d(str, (short)ercd, 3);
    print((B *)err_str);
    print((B *)":(");
    print((B *)str);
    print((B *)")\r\n");
}

/*****************************************************************************
* ݒp[^̕\
*
******************************************************************************/

int disp_param()
{
    print((B *)"[Ethernet Address   ] : [");
    print_mac(ethernet_addr);
    print((B *)"]\r\n");
    print((B *)"[Default IP Address ] : [");
    print_ip(default_ipaddr);
    print((B *)"]\r\n");
    print((B *)"[Default Gateway    ] : [");
    print_ip(default_gateway);
    print((B *)"]\r\n");
    print((B *)"[Subnet Mask        ] : [");
    print_ip(subnet_mask);
    print((B *)"]\r\n");
    return 1;
}

/*****************************************************************************
* DHCP̃eXg
*
******************************************************************************/

extern T_TERMINAL *current_terminal;

ER dhcp_command()
{
    ER ercd;

    current_terminal = (T_TERMINAL *)&console;

    ercd = dhcp_get_data(0);
    if (ercd != E_OK)
        error((char *)"\r\n\r\nError get data from DHCP server.", ercd);
    else {
        print((B *)"\r\nSuccess get data from DHCP server.\r\n");
        disp_param();
        print((B *)"[DNS IP Address     ] : [");
        print_ip(dns_ipaddr);
        print((B *)"]\r\n");
        print((B *)"[DHCP IP Address    ] : [");
        print_ip(dhcp_ipaddr);
        print((B *)"]\r\n");
    }
    current_terminal = NULL;

    return ercd;
}

/*****************************************************************************
* DNS̃eXg
******************************************************************************/

int dns_command(int argc, char *argv[])
{
    ER ercd;
    UW dns_ip, ipaddr;
    B buf[20];

    if (argc < 2) {
        print((B *)"Usage: dns domainname [dns server address]\r\n");
        return 0;
    }

    dns_ip = byte4_to_long(dns_ipaddr);
    if (dns_ip == 0)
        dns_ip = ascii_to_ipaddr((B *)argv[2]);

    if (dns_ip == 0 ){
        print((B *)"DNS server address isn't specified\r\n");
        print((B *)"Usage: dns domainname dns-server-address\r\n");
        return 0;
    }

    ercd = dns_resolver(0, dns_ip, argv[1], &ipaddr);
    if (ercd == E_OK){
        print((B *)"[");
        print((B *)argv[1]);
        print((B *)"] : [");
        long_to_byte4((UB *)buf, ipaddr);
        print_ip((UB *)buf);
        print((B *)"]\r\n");
    }
    else{
        error((char *)"Error dns_resolver", ercd);
        return 0;
    }
    return 1;
}

/* MiNET-7042łEthenet AddressIP AddressEEPROM擾܂ */

#ifndef MiNET7042
/*****************************************************************************
* Ethernet Address ̓
*
******************************************************************************/

BOOL read_ethernet_addr(void)
{
    ethernet_addr[0] = 0x00;
    ethernet_addr[1] = 0x01;
    ethernet_addr[2] = 0x4A;
    ethernet_addr[3] = 0x1C;
    ethernet_addr[4] = 0x87;
    ethernet_addr[5] = 0xE4;
    return TRUE;
}

/*****************************************************************************
* IP Address ̓
*
******************************************************************************/

BOOL read_ip_addr(void)
{
    default_ipaddr[0] = 192;
    default_ipaddr[1] = 168;
    default_ipaddr[2] = 0;
    default_ipaddr[3] = 2;

/* [^݂ꍇɁAȉLɂĂB */
    default_gateway[0] = 192;
    default_gateway[1] = 168;
    default_gateway[2] = 0;
    default_gateway[3] = 1;

    subnet_mask[0] = 255;
    subnet_mask[1] = 255;
    subnet_mask[2] = 255;
    subnet_mask[3] = 0;
    return TRUE;
}
#endif

/*****************************************************************************
* C^XN
*
******************************************************************************/
#if defined(NORTiSIM)
unsigned long RamDiskBuf[0x80000/sizeof(long)];
#endif

TASK MainTask(void)
{
    ER ercd;
    INT i;

    /* VA LAN pPbgeo */

  #ifdef DUMP
    ercd = ini_sio(1, "38400 B8 PN S1");
    if (ercd != E_OK)
        goto END;
    ercd = ctl_sio(1, TSIO_RXE|TSIO_TXE|TSIO_RTSON);
    if (ercd != E_OK)
        goto END;
    ercd = landump_ini(0, 0, 0);
    if (ercd != E_OK)
        goto END;
  #endif

    /* IP & Ethernet Address ̓ǂݏo */
  #ifndef DHCP
    read_ip_addr();
  #endif
    read_ethernet_addr();

    /* vgRX^bN */
/*
    ercd = tcp_ini();
    if (ercd != E_OK)
        goto END;

    /* R\[ */
/*
    ercd = console_ini(&console, 0, 0, 0, (B *)"9600 B8 PN S1 XON");
    if (ercd != E_OK)
        goto END;

    /* DHCPT[o[IPAhX擾 */
  #ifdef DHCP
    dly_tsk(2200/MSEC); /* n[hɂĂ͂ƏȒl(10ȏ) */
/*
    ercd = dhcp_command();
    if (ercd != E_OK)
        read_ip_addr();
*/
  #endif

    /* t@CVXe RAM fBXN̏ */
    ercd = file_ini(file, NFILE, 0, 0);
    if (ercd != E_OK)
        goto END;

  #if (defined(MS7709A) || defined(MS7709) || defined(MS7750))
    /* () EDO-DRAM ̌㔼 1MB (AC100000`AC1FFFFF) gp܂.  */
    /* ̗̈ɕϐX^bN蓖ĂȂ悤ӂĂ. */
    ercd = disk_ini(disk, "A:", ramdisk, 0xac100000, 0x100000);
    if (ercd != E_OK)
        goto END;
#elif defined(MiNET7042)
    /* () SRAM ̌㔼 512KB (0x480000`0x4FFFFF) gp܂.  */
    /* ̗̈ɕϐX^bN蓖ĂȂ悤ӂĂ. */
    ercd = disk_ini(disk, "A:", ramdisk, 0x480000, 0x80000);
    if (ercd != E_OK)
        goto END;
#elif defined(MS7615) || defined(SH7615EV)
    /* () SDRAM ̌㔼 1MB (0x06F00000`0x06FFFFFF) gp܂.  */
    /* ̗̈ɕϐX^bN蓖ĂȂ悤ӂĂ. */
    ercd = disk_ini(disk, (char *)"A:", ramdisk, 0x06F00000, 0x100000);
    if (ercd != E_OK)
        goto END;
  #elif defined(NORTiSIM)
    ercd = disk_ini(disk, (char *)"A:", ramdisk, (UW)RamDiskBuf, sizeof(RamDiskBuf));
    if (ercd != E_OK)
        goto END;
  #endif

    /* UDP GR[T[o[ */
/*
    ercd = udpecho_ini(0, 0);
    if (ercd != E_OK)
        goto END;
*/

    /* TFTP T[o[ */
/*
    ercd = tftpsrv_ini(0, 0);
    if (ercd != E_OK)
        goto END;
*/

    /* TCP GR[T[o[ */
/*
    ercd = tcpecho_ini(0, 0, 0);
    if (ercd != E_OK)
        goto END;
*/
    
    /* TELNET f[ */
/*
    ercd = telnetd_ini(&telnetd, 0, 0, 0, 0);
    if (ercd != E_OK)
        goto END;
    ercd = shell_ini(&telnetd, 0, 0, telnet_passwd_check);
    if (ercd != E_OK)
        goto END;

    ercd = shell_ini(&console, 0, 0, telnet_passwd_check);
    if (ercd != E_OK)
        goto END;
*/

    /* FTP T[o[ */
/*
    ercd = ftp_cre_srv(&ftp, 0, 0, 0, 0, NULL);
    if (ercd != E_OK)
        goto END;
    ercd = ftp_sta_srv(&ftp);
    if (ercd != E_OK)
        goto END;
*/

END:
  #if defined(NORTiSIM)
    if (ercd == E_OK) {         /* 1b7segJEgAbv */
        for (i = 0;;) {
            tslp_tsk(1000/MSEC);
            put_7seg((i++) & 0xf);
        }
    } else {
        put_7seg( 0xe );        /* Error */
        for (;;)
            slp_tsk();
    }
  #else
    for (;;)
        slp_tsk();
  #endif
}

#if (defined(MS7709A) || defined(MS7750) || defined(MS7615) || defined(SH7615EV))
/*****************************************************************************
* SuperIO ̏
*
******************************************************************************/

#if defined(MS7615)
#define  CONF_BASE  (*(volatile unsigned short *) 0x22000700)
#define  CONF_PORT  (*(volatile unsigned short *) 0x220007E0)
#define  INDEX_PORT (*(volatile unsigned short *) 0x220007E0)
#define  DATA_PORT  (*(volatile unsigned short *) 0x220007E2)
#else
#define  CONF_BASE  (*(volatile unsigned short *) 0xB0400700)
#define  CONF_PORT  (*(volatile unsigned short *) 0xB04007E0)
#define  INDEX_PORT (*(volatile unsigned short *) 0xB04007E0)
#define  DATA_PORT  (*(volatile unsigned short *) 0xB04007E2)
#endif

static unsigned short  read_w(unsigned short  index)
{
    INDEX_PORT = index;
    return  DATA_PORT;
}

static void  write_w(unsigned short  index, unsigned short  data)
{
    INDEX_PORT = index;
    DATA_PORT  = data;
}

void    com1_conf(unsigned short add)
{
unsigned short low_add;
unsigned short high_add;

    low_add = ((add << 8) & 0xff00);
    high_add = (add & 0xff00);

    CONF_PORT  = 0x5500;  /* Entering the configuration state */
    CONF_PORT  = 0x5500;

    write_w(0x0700, 0x0400);
    write_w(0x3000, 0x0100);
    write_w(0x6000, high_add);
    write_w(0x6100, low_add);
    write_w(0x7000, 0x0400);   /* IRQ4 */
    write_w(0xF000, 0x0000);
    write_w(0x0700, 0x0000);

    CONF_PORT  = 0xAA00;  /* Exiting the configuration state */

}

void    com2_conf(unsigned short add)
{
unsigned short low_add;
unsigned short high_add;

    low_add  = ((add << 8) & 0xff00);
    high_add = (add & 0xff00);

    CONF_PORT  = 0x5500;  /* Entering the configuration state */
    CONF_PORT  = 0x5500;

    write_w(0x0700, 0x0500);
    write_w(0x3000, 0x0100);
    write_w(0x6000, high_add);
    write_w(0x6100, low_add);
    write_w(0x7000, 0x0300);   /* IRQ3 */
    write_w(0xF000, 0x0000);
    write_w(0x0700, 0x0000);

    CONF_PORT  = 0xAA00;  /* Exiting the configuration state */
}

/*****************************************************************************
* peripheral ̗
*
******************************************************************************/

void ini_pio(void)
{
 #if !defined(SH7615EV)
    com1_conf(0x03f8);      /* UART0 = 0x220007F0   */
    com2_conf(0x02f8);      /* UART1 = 0x220005F0   */
 #endif

 #if defined(MS7615) || defined(SH7615EV)
  #if !defined(SH7615EV)
    sfr_setw(PFC_PBCR, PBCR_RXD1);  /* to RxD1 */
    sfr_setw(PFC_PBCR, PBCR_TXD1);  /* to TxD1 */
    sfr_setw(PFC_PBCR, PBCR_RTS1);  /* to ~RTS */
    sfr_setw(PFC_PBCR, PBCR_CTS1);  /* to ~CTS */
  #endif

    sfr_setw(PFC_PBCR2, PBCR_RXD2);  /* to RxD2 */
    sfr_setw(PFC_PBCR2, PBCR_TXD2);  /* to TxD2 */
 #endif
}
#endif /* (defined(MS7709A) || defined(MS7750) || defined(MS7615) || defined(SH7615EV)) */

/*****************************************************************************
*  main֐
*
*  RtBO[^gpꍇ́A
*  RtBO[^쐬t@CƃNĂB
******************************************************************************/
#ifndef USE_CONFIG
int main(void)
{
    ID id;

    /* VXe */
  #ifdef MiNET7042
    ini_port();                 /* |[g */
    ini_eeprom();               /* EEPROM |[g */
  #elif (defined(MS7709A) || defined(MS7750) || defined(MS7615) || defined(SH7615EV))
    ini_pio();                  /* peripheral  */
  #endif
    sysini();

    /* ^XN */

    id = acre_tsk(&cMainTask);

    /* ^XNN */

    sta_tsk(id, 0);

    /* VXeN */

    intsta();                   /* ^C}݂N */
    syssta();                   /* }`^XNֈڍs */
}
#endif

/* end */
